<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Api;

use Exception;
use Rebilly\Entities\Product;
use Rebilly\Rest\Collection;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

/**
 * Class ProductService
 * @package Rebilly\Wp\Member\Api
 */
class ProductService extends Service
{
    /**
     * @param string $id
     * @param int $cache
     * @param bool $force
     *
     * @return Product|null
     */
    public function get(string $id, int $cache = 5 * 60, bool $force = false): ?Product
    {
        return $this->cached(
            "product-$id",
            function () use ($id) {
                try {
                    return $this->api()
                        ->products()
                        ->load($id);
                } catch (Exception $ex) {
                }

                return false;
            },
            $cache,
            $force
        ) ?: null;
    }

    /**
     * @param string[]|array $ids
     * @param bool $only
     * @param int $cache
     * @param bool $force
     *
     * @return Product[]|Collection
     */
    public function all(array $ids = [], bool $only = false, int $cache = 5 * 60, bool $force = false): Collection
    {
        $key = 'products';
        $params = [];
        if ($only === true || count($ids) > 0) {
            $idList = implode(',', $ids);
            $params['filter'] = "id:$idList";
            $key .= '-' . sha1($idList);
        }

        return $this->cached(
            $key,
            function () use ($params) {
                try {
                    return $this->api()
                        ->products()
                        ->search($params);
                } catch (Exception $ex) {
                }

                return false;
            },
            $cache,
            $force
        ) ?: new Collection(new Product());
    }

    public function allNames(array $ids = [], bool $only = false, int $cache = 5 * 60, bool $force = false): array
    {
        $names = [];
        $products = $this->all($ids, $only, $cache, $force);
        foreach ($products as $product) {
            $names[$product->getId()] = $product->getName();
        }

        return $names;
    }
}
