<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Api;

use Exception;
use Rebilly\Entities\Website;
use Rebilly\Rest\Collection;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

/**
 * Class WebsiteService
 * @package Rebilly\Wp\Member\Api
 */
class WebsiteService extends Service
{
    /**
     * @param string[]|array $ids
     * @param bool $only
     * @param int $cache
     * @param bool $force
     *
     * @return Website[]|Collection
     */
    public function all(array $ids = [], bool $only = false, int $cache = 5 * 60, bool $force = false): Collection
    {
        $key = 'websites';
        $params = [];
        if ($only === true || count($ids) > 0) {
            $idList = implode(',', $ids);
            $params['filter'] = "id:$idList";
            $key .= '-' . sha1($idList);
        }

        return $this->cached(
            $key,
            function () use ($params) {
                try {
                    return $this->api()
                        ->websites()
                        ->search($params);
                } catch (Exception $ex) {
                }

                return false;
            },
            $cache,
            $force
        ) ?: new Collection(new Website());
    }

    public function allNames(array $ids = [], bool $only = false, int $cache = 5 * 60, bool $force = false): array
    {
        $names = [];
        $websites = $this->all($ids, $only, $cache, $force);
        foreach ($websites as $website) {
            $names[$website->getId()] = $website->getName();
        }

        return $names;
    }

    public function getFromAll(string $id): ?Website
    {
        $all = $this->all();
        foreach ($all as $website) {
            if ($website->getId() === $id) {
                return $website;
            }
        }

        return null;
    }
}
