<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Entity;

use Rebilly\Wp\Member\Reformer\Node;
use Rebilly\Wp\Member\Reformer\Reformer;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

trait HasDomTree
{
    protected $traitAttributes = ['for', 'name', 'value', 'type', 'required', 'placeholder', 'checked', 'data-geo-fill'];

    /**
     * @param array $root
     *
     * @return array
     */
    public function buildComponents(array $root = []): array
    {
        if (!isset($root['type'])) {
            return array_map([$this, 'buildComponent'], $root);
        }

        return $this->buildComponent($root);
    }

    /**
     * @param array $component
     *
     * @return array
     */
    public function buildComponent(array $component): array
    {
        $type = $component['type'] ?? 'paragraph';

        $attributes = $component['attributes'] ?? [];
        $id = $attributes['id'] ?? '';
        unset($attributes['id']);

        $classes = implode(' ', array_filter(array_map(function ($class) {
            return $class['name'] ?? false;
        }, $component['classes'])));

        foreach ($component['attributes'] ?? [] as $key => $value) {
            if (in_array($key, ['required', 'checked'], true) && !in_array($value, [false, 'false'], true)) {
                $attributes[$key] = true;
            } elseif(!blank($value)) {
                $attributes[$key] = $value;
            }
        }

        $traits = [];
        foreach ($component['traits'] ?? [] as $trait) {
            if (blank($trait['name'] ?? '') || blank($trait['value'] ?? '')) {
                continue;
            }
            [$key, $value] = [$trait['name'], $trait['value']];
            if ($key === 'id') {
                $id = $value;
            } elseif (in_array($key, $this->traitAttributes, true)) {
                $attributes[$key] = $value;
            } else {
                $traits[$key] = $value;
            }
        }
        foreach (['rebilly', 'addon', 'bump', 'data-preset', 'condition', 'options', 'plan'] as $name) {
            if (isset($attributes[$name])) {
                $traits[$name] = $attributes[$name];
                unset($attributes[$name]);
            }
        }
        $content = $type === 'textnode' ? $component['content'] ?? '' : '';
        if ($component['type'] !== 'select' && !blank($component['content'] ?? '')) {
            $children = [
                [
                    'type' => 'textnode',
                    'content' => $component['content'],
                ],
            ];
        } else {
            $children = $this->buildComponents($component['components'] ?? []);
        }

        return [
            'type' => $type ?: $component['tagName'],
            'tag' => $component['tagName'],
            'id' => $id,
            'class' => $classes,
            'attributes' => $attributes,
            'content' => $content,
            'children' => $children,
            'settings' => $traits,
        ];
    }

    /**
     * @param array $components
     *
     * @return array|Node[]
     */
    public function buildDom(array $components): array
    {
        return Reformer::buildNodes($components);
    }
}
