<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Entity;

use WP_User;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

trait HasMeta
{
    /**
     * @param bool $byName
     *
     * @return array
     */
    public function metaAll(bool $byName = false): array
    {
        $result = [];
        foreach (static::$meta as $metaName => $metaKey) {
            $value = get_metadata($this->metaType(), $this->getKey(), $metaKey, true);
            if ($value !== false && $value !== '' && $value !== []) {
                $result[$byName ? $metaName : $metaKey] = $value;
            }
        }

        return wp_parse_args($result, static::metaDefaults());
    }

    private function metaType(): string
    {
        return $this instanceof WP_User ? 'user' : 'post';
    }

    public function getKey(): ?int
    {
        return ((int) $this->ID) ?: null;
    }

    protected static function metaDefaults()
    {
        /** @noinspection PhpUndefinedFieldInspection */
        return property_exists(static::class, 'metaDefaults') ? static::$metaDefaults : [];
    }

    public function metaGet(string $key, bool $byName = false, $default = false)
    {
        if ($byName) {
            $key = static::metaKey($key);
        }

        $meta = get_metadata($this->metaType(), $this->getKey(), $key, true);

        return $meta === '' ? $default : $meta;
    }

    public static function metaKey(string $name = '')
    {
        return $name === '' ? static::$meta : (static::$meta[$name] ?? false);
    }

    public function metaMap(array $values, bool $byName = false): array
    {
        $map = [];
        $metaNames = array_keys(static::$meta);
        $metaKeys = array_values(static::$meta);
        $metaByKey = array_flip(static::$meta);
        foreach ($values as $k => $value) {
            if (is_integer($k)) {
                $key = ($byName ? $metaNames : $metaKeys)[$k];
            } elseif ($byName) {
                $key = $metaByKey[$k];
            } else {
                $key = static::$meta[$k];
            }
            $map[$key] = $value;
        }

        return $map;
    }

    public function metaUpdate(array $values, bool $byName = false): void
    {
        $type = $this instanceof WP_User ? 'user' : 'post';
        foreach (static::$meta as $metaName => $metaKey) {
            $key = $byName ? $metaName : $metaKey;
            if (array_key_exists($key, $values)) {
                update_metadata($type, $this->getKey(), $metaKey, $values[$key]);
            }
        }
    }
}
