<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Entity;

use WP_Post;
use WP_User;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

/**
 * Class RestrictableType
 * @package Rebilly\Wp\Member
 *
 * @mixin WP_Post
 */
class RestrictableType
{
    use PostProxy, HasMeta;

    protected static $meta = [
        'status' => '_rebilly_m_status',
        'capabilities' => '_rebilly_m_capabilities',
        'members' => '_rebilly_m_members',
    ];

    /**
     * @param WP_User|User|int|null $user
     *
     * @return bool
     */
    public function canAccess($user = null): bool
    {
        if (!$this->isProtected()) {
            return true;
        }
        if (!$user = User::find($user)) {
            return false;
        }

        return $user->canOverride($this) ||
            $user->isCapable($this->getRestrictions());
    }

    /**
     * @return bool
     */
    public function isProtected(): bool
    {
        return (bool) $this->metaGet('status', true, false);
    }

    /**
     * @return array
     */
    public function getRestrictions(): array
    {
        return (array) $this->metaGet('capabilities', true, []);
    }

    /**
     * @return array
     */
    public function getMembers(): array
    {
        return $this->metaGet('members', true, []);
    }
}
