<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Exception;

use Throwable;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

class AjaxResponseException extends \Exception
{
    public const SUCCESS = 'success';

    public const FAIL = 'fail';

    /** @var string */
    protected $type;

    /** @var array|null */
    protected $errors;

    /** @var array */
    protected $additional;

    /** @var bool */
    protected $hasMessage;

    public function __construct(string $type, ?string $message = '', ?array $errors = null, array $additional = [], int $code = 0, Throwable $previous = null)
    {
        parent::__construct($message, $code, $previous);
        $this->type = $type;
        $this->errors = $errors;
        $this->additional = $additional;
        $this->hasMessage = $message !== null;
    }

    /**
     * @return bool
     */
    public function isSuccess(): bool
    {
        return $this->type === self::SUCCESS;
    }

    /**
     * @return string
     */
    public function getType(): string
    {
        return $this->type;
    }

    /**
     * @param string $type
     */
    public function setType(string $type): void
    {
        $this->type = $type;
    }

    /**
     * @return array|null
     */
    public function getErrors(): ?array
    {
        return $this->errors;
    }

    /**
     * @param array|null $errors
     */
    public function setErrors(?array $errors): void
    {
        $this->errors = $errors;
    }

    /**
     * @return array
     */
    public function getAdditional(): array
    {
        return $this->additional;
    }

    /**
     * @param array $additional
     */
    public function setAdditional(array $additional): void
    {
        $this->additional = $additional;
    }

    /**
     * @return bool
     */
    public function isHasMessage(): bool
    {
        return $this->hasMessage;
    }
}
