<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

if (!function_exists('blank')) {
    function blank()
    {
        foreach (func_get_args() as $str) {
            if ($str === null ||
                (is_bool($str) && !$str) ||
                (is_array($str) && count($str) === 0) ||
                (is_string($str) && trim($str) === '')) {
                return true;
            }
        }

        return false;
    }
}

if (!function_exists('renderIsolate')) {
    /**
     * @param string $file
     * @param mixed $data
     * @param bool $return
     * @return string|null
     */
    function renderIsolate($file, $data = [], $return = false)
    {
        if ($return) {
            ob_start();
        }
        extract($data);
        /** @noinspection PhpIncludeInspection */
        include $file;
        if ($return) {
            return ob_get_clean();
        }

        return null;
    }
}

if (!function_exists('wp_rebilly_m_cached')) {
    /**
     * Saves callback result as a transient value
     *
     * @param $key
     * @param false|callable $callback If is set to false will delete cached value
     * @param int $expires Time in seconds. 0 if no limits
     *
     * @return bool|mixed
     */
    function wp_rebilly_m_cached($key, $callback, $expires = 0)
    {
        $transient = "wp_rebilly_m__{$key}";
        if ($callback === false) {
            delete_transient($key);

            return true;
        }

        $value = get_transient($transient);
        if ($value === false) {
            $value = call_user_func($callback);
            set_transient($transient, $value, $expires);
        }

        return $value;
    }
}

if (!function_exists('currencySign')) {
    /**
     * @param string $currency
     * @param string|null $locale
     * @return string
     */
    function currencySign(string $currency, ?string $locale = null): string
    {
        $locale = $locale ?? 'en-US';
        $fmt = new NumberFormatter($locale . "@currency=$currency", NumberFormatter::CURRENCY);

        return $fmt->getSymbol(NumberFormatter::CURRENCY_SYMBOL);
    }
}

if (!function_exists('rebilly_money')) {
    /**
     * @param float $value
     * @param string $currency
     * @param string|null $locale
     * @return string
     */
    function rebilly_money(float $value, string $currency, string $locale = null): string
    {
        $locale = $locale ?? get_user_locale();
        $fmt = new NumberFormatter($locale, NumberFormatter::CURRENCY);

        return $fmt->formatCurrency($value, $currency);
    }
}
