<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Integration\BeaverBuilder\RebillyForm;

use Rebilly\Wp\Member\Entity\Form;
use Rebilly\Wp\Member\Integration\BeaverBuilder\Module;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

class FormModule extends Module
{
    /** @var array */
    protected static $forms = [];

    public function __construct()
    {
        parent::__construct([
            'name' => __('Form', REBILLY_WP_LANG_DOMAIN),
            'description' => __('Insert a Rebilly form on your page', REBILLY_WP_LANG_DOMAIN),
            'group' => __('Rebilly', REBILLY_WP_LANG_DOMAIN),
            'category' => __('Basic Modules', 'fl-builder'),
            'dir' => REBILLY_WP_DIR . 'template/admin/integration/beaver-builder/form/',
            'url' => REBILLY_WP_URL . 'template/admin/integration/beaver-builder/form/',
            'icon' => 'icon.svg',
        ]);
    }

    protected static function settings(): array
    {
        return [
            'general1' => [
                'title' => __('General', REBILLY_WP_LANG_DOMAIN),
                'sections' => [
                    'settings' => [
                        'title' => __('Settings', REBILLY_WP_LANG_DOMAIN),
                        'fields' => [
                            'formId' => [
                                'type' => 'select',
                                'label' => __('Form', REBILLY_WP_LANG_DOMAIN),
                                'default' => '',
                                'options' => static::getForms(),
                            ],
                            'condition' => [
                                'type' => 'select',
                                'label' => __('Condition', REBILLY_WP_LANG_DOMAIN),
                                'default' => '',
                                'options' => static::getConditions(),
                            ],
                        ],
                    ],
                ],
            ],
        ];
    }

    public static function getForms(): array
    {
        if (count(static::$forms) === 0) {
            $formsRaw = Form::all() ?? [];
            static::$forms = [
                '' => __('Select a Form', REBILLY_WP_LANG_DOMAIN),
            ];
            foreach ($formsRaw as $form) {
                static::$forms[$form->ID] = get_the_title($form);
            }
        }

        return static::$forms;
    }

    public static function getConditions(): array
    {
        return [
            '' => __('Display for all', REBILLY_WP_LANG_DOMAIN),
            'only:auth' => __('Members only', REBILLY_WP_LANG_DOMAIN),
            'only:guest' => __('Guests only', REBILLY_WP_LANG_DOMAIN),
        ];
    }
}
