<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Reformer;

use Rebilly\Wp\Member\Reformer\Traits\HasAttributes;
use Rebilly\Wp\Member\Reformer\Traits\HasChildren;
use Rebilly\Wp\Member\Reformer\Traits\HasDomRenderer;
use Rebilly\Wp\Member\Reformer\Traits\HasShowFor;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

class Block implements Node
{
    use HasAttributes, HasChildren, HasDomRenderer, HasShowFor;

    protected $tag = '';

    protected $config = [];

    /**
     * Label constructor.
     * @param array $config
     */
    public function __construct(array $config)
    {
        if (!in_array($config['tag'], ['div', 'section', 'article'], true)) {
            $config['tag'] = 'div';
        }

        $this->config = $config;

        $this->tag = $config['tag'];

        $this->children = Reformer::buildNodes($config['children'] ?? []);

        $this->attributes['id'] = $config['id'];
        $this->attributes['class'] = $config['class'];

        $condition = $config['settings']['condition'] ?? '';
        if (!blank($condition)) {
            $this->attributes['data-condition'] = !is_string($condition)? json_encode($condition) : $condition;
        }
    }

    public function getTag(): string
    {
        return $this->tag;
    }
}
