<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Reformer;

use Rebilly\Wp\Member\Reformer\Traits\HasAttributes;
use Rebilly\Wp\Member\Reformer\Traits\HasChildren;
use Rebilly\Wp\Member\Reformer\Traits\HasDomRenderer;
use Rebilly\Wp\Member\Reformer\Traits\HasShowFor;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

class Link implements Node
{
    use HasAttributes, HasChildren, HasDomRenderer, HasShowFor;

    protected $config = [];

    /**
     * Label constructor.
     * @param array $config
     */
    public function __construct(array $config)
    {
        $this->config = $config;

        $this->children = Reformer::buildNodes($config['children'] ?? []);

        foreach ($config['attributes'] as $key => $value) {
            if (blank($value)) {
                continue;
            }
            if (in_array($key, ['href', 'target', 'title'], true)) {
                $this->attributes[$key] = $value;
            }
        }
    }

    public function getTag(): string
    {
        return 'a';
    }
}
