<?php
/**
 * This source file is proprietary and part of Rebilly.
 *
 * (c) Rebilly SRL
 *     Rebilly Ltd.
 *     Rebilly Inc.
 *
 * @see https://www.rebilly.com
 */

declare(strict_types=1);

namespace Rebilly\Wp\Member\Reformer;

// Exit if accessed directly.
if (!defined('ABSPATH')) {
    exit();
}

class Reformer
{
    protected static $map = [
        'form' => Form::class,
        'form-group' => FormGroup::class,
        'label' => Label::class,
        'input' => Input::class,
        'number' => Number::class,
        'checkbox' => Input::class,
        'radio' => Input::class,
        'select' => Select::class,
        'option' => Option::class,
        'textarea' => Textarea::class,
        'button' => Button::class,
        'framepay' => Framepay::class,

        'header' => Header::class,
        'paragraph' => Paragraph::class,
        'divider' => Divider::class,
        'link' => Link::class,
        'text' => Text::class,
        'textnode' => TextNode::class,
    ];

    protected static $defaultNode = Block::class;

    /**
     * @param array $nodes
     *
     * @return Node[]|array
     */
    public static function buildNodes(array $nodes): array
    {
        if (isset($nodes['type'])) {
            $nodes = [$nodes];
        }

        return array_filter(array_map([static::class, 'buildNode'], $nodes));
    }

    /**
     * @param array $config
     *
     * @return Node|null
     */
    public static function buildNode(array $config): ?Node
    {
        $type = $config['type'] ?? '';
        if (array_key_exists($type, static::$map)) {
            return new static::$map[$type]($config);
        }

        return new static::$defaultNode($config);
    }

    /**
     * @param array $components
     * @param array $env
     *
     * @return string
     */
    public static function renderComponents(array $components, array $env): string
    {
        return static::renderNodes(static::buildNodes($components), $env);
    }

    /**
     * @param Node[]|array $nodes
     * @param array $env
     *
     * @return string
     */
    public static function renderNodes(array $nodes, array $env): string
    {
        return implode('', array_map(function (Node $node) use ($env) {
            return $node->render($env);
        }, $nodes));
    }

    /**
     * @param array $components
     *
     * @return string
     */
    public static function renderText(array $components): string
    {
        if (isset($components['type'])) {
            $components = [$components];
        }
        $text = '';
        foreach ($components as $component) {
            if (($component['type'] ?? '') === 'textnode' || !blank($component['content'])) {
                $text .= $component['content'] ?? '';
            } else {
                $text .= static::renderText($component['children'] ?? []);
            }
        }

        return $text;
    }
}
