<?php
/**
 * @var array $subscriptions
 * @var \Rebilly\Wp\Member\Action\AjaxAction[]|array $actions
 */
?>
<?php if (count($subscriptions) > 0): ?>
    <?php foreach ($subscriptions as $subscriptionId => $subscription): ?>
        <div class="rebilly_subscription_wrap">
            <div class="rebilly_subscription" data-id="<?php esc_attr_e($subscriptionId); ?>"
                 data-nonce="<?php esc_attr_e($actions['cancel']->generateNonce($subscriptionId)); ?>">
                <ul class="plans">
                    <?php foreach ($subscription['plans'] as $plan): ?>
                        <li class="plan">
                    <span class="title">
                        <?php esc_html_e($plan['name']); ?>
                    </span>
                            <span class="quantity">
                        <?php esc_html_e($plan['quantity']); ?>
                    </span>
                            <span class="price">
                        <?php esc_html_e(rebilly_money($plan['price'], $plan['currency'])); ?>
                    </span>
                            <?php if ($plan['recurring'] !== false): ?>
                                <span class="recurring">
                        <?php $plan['recurring']['length'] > 1 && esc_html_e($plan['recurring']['length']); ?>
                        <?php esc_html_e($plan['recurring']['unit']); ?>
                    </span>
                            <?php endif; ?>
                        </li>
                    <?php endforeach; ?>
                </ul>
                <ul class="date">
                    <?php if ($subscription['createdTime'] !== $subscription['startTime']): ?>
                        <li>
                            <?php esc_html_e('Created at:', REBILLY_WP_LANG_DOMAIN); ?>
                            <div class="text-muted">
                                <?php esc_html_e(date_i18n(get_option('date_format'), $subscription['createdTime'])); ?>
                            </div>
                        </li>
                    <?php endif; ?>
                    <li>
                        <?php esc_html_e('Started at:', REBILLY_WP_LANG_DOMAIN); ?>
                        <div class="text-muted">
                            <?php esc_html_e(date_i18n(get_option('date_format'), $subscription['startTime'])); ?>
                        </div>
                    </li>
                    <?php if ($subscription['canceledTime'] !== false): ?>
                        <li>
                            <?php esc_html_e('Canceled at:', REBILLY_WP_LANG_DOMAIN); ?>
                            <div class="text-muted">
                                <?php esc_html_e(date_i18n(get_option('date_format'), $subscription['canceledTime'])); ?>
                            </div>
                        </li>
                    <?php else: ?>
                        <li>
                            <?php esc_html_e('Will be renewed at:', REBILLY_WP_LANG_DOMAIN); ?>
                            <div class="text-muted">
                                <?php esc_html_e(date_i18n(get_option('date_format'), $subscription['renewalTime'])); ?>
                            </div>
                        </li>
                    <?php endif; ?>
                </ul>
                <div class="status <?php esc_attr_e($subscription['status']); ?>">
                    <?php esc_html_e(mb_convert_case($subscription['status'], MB_CASE_TITLE)); ?>
                </div>
                <div class="actions">
                    <?php if (in_array($subscription['status'], ['active', 'pending'], true)): ?>
                        <button class="action cancel wp_rebilly_m__subscription__cancel"
                                title="<?php esc_attr_e('Cancel', REBILLY_WP_LANG_DOMAIN); ?>">
                            <?php echo file_get_contents(REBILLY_WP_DIR . 'images/icon/cancel.svg'); ?>
                        </button>
                    <?php endif; ?>
                </div>
            </div>
        </div>
    <?php endforeach; ?>
<?php else: ?>
    <div class="rebilly_notice">
        <?php esc_html_e('You have no active subscriptions yet.', REBILLY_WP_LANG_DOMAIN); ?>
    </div>
<?php endif; ?>
