<?php
/**
 * @var User $user
 * @var string $defaultCardId
 * @var PaymentCard $card
 * @var array|Action[] $actions
 */

use Rebilly\Entities\PaymentCard;
use Rebilly\Wp\Member\Action\Action;
use Rebilly\Wp\Member\User;

$brandIcons = [
    'Visa' => 'visa/visa-card-dark.svg',
    'American Express' => 'amex/amex-dark.svg',
    'MasterCard' => 'mastercard/mastercard-dark.svg',
    'Discover' => 'discover/discover-card-dark.svg',
    'Maestro' => 'maestro/maestro-card-dark.svg',
    'Solo' => 'switch/switch-card-dark.svg',
    'Electron' => 'visa/visa-card-dark.svg',
    'JCB' => 'jcb/jcb-card-dark.svg',
    'Voyager' => 'visa/visa-card-dark.svg',
    'Diners Club' => 'diners-club/diners-club-dark.svg',
    'Switch' => 'switch/switch-card-dark.svg',
    'Laser' => 'laser/laser-card-dark.svg',
    'China UnionPay' => 'unionpay/unionpay-card-dark.svg',
    'AstroPay Card' => 'visa/visa-card-dark.svg',
    'Fuel Credit Card' => 'visa/visa-card-dark.svg',
];

$cardId = $card->getId();
$isDefault = $cardId == $defaultCardId;
$status = $card->getStatus();
$classList = $status . ' ' . ($isDefault ? 'primary' : '');
$bankName = $card->getBankName();
$bankCountry = $card->getBankCountry();
$bin = $card->getBin();
$cardNumber = [
    substr($bin, 0, 4),
    substr($bin, 4, 2) . '**',
    '****',
    $card->getLast4(),
];
$expMonth = $card->getExpMonth();
$expYear = substr($card->getExpYear(), -2);
$exp = (strlen($expMonth) < 2 ? '0' : '') . $expMonth . '/' . $expYear;
$fullName = trim($card->getBillingAddress()->getFirstName() . ' ' . $card->getBillingAddress()->getLastName());
$brand = $card->getBrand();
$brandIcon = REBILLY_WP_URL . 'images/payment-instruments/' . ($brandIcons[$brand] ?? $brandIcons['Visa']);
$countryIcon = REBILLY_WP_URL . 'images/country-flags/' . strtolower($bankCountry) . '.svg';
$nonce = [
    'update' => $actions['update']->generateNonce($cardId),
    'delete' => $actions['delete']->generateNonce($cardId),
];

?>
<div class="card <?php esc_attr_e($classList); ?>"
     data-id="<?php esc_attr_e($cardId); ?>"
     data-update-nonce="<?php esc_attr_e($nonce['update']); ?>"
     data-delete-nonce="<?php esc_attr_e($nonce['delete']); ?>">
    <div class="flip">
        <div class="front">
            <div class="strip-bottom"></div>
            <div class="strip-top"></div>
            <?php if (!blank($bankCountry)): ?>
                <div class="country">
                    <img src="<?php esc_attr_e($countryIcon); ?>" alt="<?php esc_attr_e($bankCountry); ?>">
                </div>
            <?php endif; ?>
            <div class="bank-name"><?php esc_html_e(blank($bankName) ? 'Rebilly' : $bankName); ?></div>
            <div class="chip">
                <div class="chip-line"></div>
                <div class="chip-line"></div>
                <div class="chip-line"></div>
                <div class="chip-line"></div>
                <div class="chip-main"></div>
            </div>
            <?php echo file_get_contents(REBILLY_WP_DIR . 'images/icon/wave.svg'); ?>
            <div class="card-number">
                <?php foreach ($cardNumber as $part): ?>
                    <div class="section"><?php esc_html_e($part); ?></div>
                <?php endforeach; ?>
            </div>
            <div class="end">
                <span class="end-text">exp:</span><span class="end-date"> <?php esc_html_e($exp); ?></span>
            </div>
            <div class="card-holder"><?php esc_html_e($fullName); ?></div>
            <?php if (!blank($brand)): ?>
                <div class="brand"><img src="<?php esc_attr_e($brandIcon); ?>" alt="<?php esc_attr_e($brand); ?>"></div>
            <?php endif; ?>
            <div class="status"><?php esc_html_e($status); ?></div>
            <div class="actions">
                <?php if (!$isDefault): ?>
                    <button class="action primary" data-form="primary" title="Set as default">
                        <?php echo file_get_contents(REBILLY_WP_DIR . 'images/icon/key.svg'); ?>
                    </button>
                <?php endif; ?>
                <button class="action edit" data-form="edit" title="Edit">
                    <?php echo file_get_contents(REBILLY_WP_DIR . 'images/icon/pen.svg'); ?>
                </button>
                <button class="action deactivate" data-form="deactivate" title="Delete">
                    <?php echo file_get_contents(REBILLY_WP_DIR . 'images/icon/trash.svg'); ?>
                </button>
            </div>
        </div>
        <div class="back">
            <div class="strip-black">
                <div class="wp_rebilly_m__logo show">
                    <div class="mr-re"></div>
                    <div class="mr-bil"></div>
                    <div class="mr-ly"></div>
                </div>
                <div class="validation-error"></div>
            </div>
            <div class="form primary">
                <div class="note">Are you sure?</div>
                <button class="action set-primary" title="Primary">Set as default</button>
            </div>
            <div class="form edit">
                <div class="month">
                    <label>month</label>
                    <input type="text" value="<?php esc_attr_e($expMonth); ?>" maxlength="2">
                </div>
                <div class="year">
                    <label>year</label>
                    <input type="text" value="<?php esc_attr_e($expYear); ?>" maxlength="2">
                </div>
                <button class="action update" title="Update">Update</button>
            </div>
            <div class="form deactivation">
                <div class="note">Are you sure?</div>
                <button class="action deactivate" title="Deactivate">Deactivate</button>
            </div>
        </div>
    </div>
</div>
